import sys
import os
import csv
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import copy


def main():

    prologTimes = {}

    if(len(sys.argv) < 2):
        print("Please give a test folder")
        return
    
    files = os.listdir('tests/'+sys.argv[1]+'/GeneratedExamples')

    prologTimes = {}

    for file in files:
        splitted = file.split('.')
        prologTimes.update({splitted[1]: []})

    pythonTimes = copy.deepcopy(prologTimes)
    optimizationTimes = copy.deepcopy(prologTimes)

    with open("tests/"+sys.argv[1]+"/times/prologTimes.txt") as f:
       prolog = f.read().splitlines() 

    with open("tests/"+sys.argv[1]+"/times/pythonTimes.txt") as f:
        python = f.read().splitlines()

    with open("tests/"+sys.argv[1]+"/times/optimizedTimes.txt") as f:
        optimized = f.read().splitlines()

    for l in prolog:
        splitted = l.split(".")
        if splitted[2]!= '':
            minutes = splitted[2].split("m")[0]
            seconds = splitted[2].split("m")[1].split("s")[0].replace(",", ".")
            time = (int(minutes) * 60) + float(seconds)
            print(splitted[1], minutes, seconds, time)
        else:
            time = 120
            
        prologTimes[splitted[1]].append(time)

    for l in python:
        splitted = l.split(".")
        if splitted[2]!= '':
            minutes = splitted[2].split("m")[0]
            seconds = splitted[2].split("m")[1].split("s")[0].replace(",", ".")
            time = (int(minutes) * 60) + float(seconds)
        else:
            time = 120

        pythonTimes[splitted[1]].append(time)

    for l in optimized:
        splitted = l.split(".")
        if splitted[2]!= '':
            minutes = splitted[2].split("m")[0]
            seconds = splitted[2].split("m")[1].split("s")[0].replace(",", ".")
            time = (int(minutes) * 60) + float(seconds)
        else:
            time = 120

        optimizationTimes[splitted[1]].append(time)

    print("prolog", prologTimes['7_5_20'])
    print("python", pythonTimes['7_5_20'])
    print("optimized", optimizationTimes['7_5_20'])

    prologList = list(prologTimes.items())
    pythonList = list(pythonTimes.items())
    optimizationList = list(optimizationTimes.items())

    with open("tests/"+sys.argv[1]+"/times/tiempos.csv", mode='w', newline='') as file:
        writer = csv.writer(file)
        writer.writerow(["ejecución", "programa", "ejemplo", "tiempo"])
        for t in prologList:
            for i in range(0, 5):
                if len(t[1]) != 0:
                    writer.writerow([i+1, "prolog", t[0], t[1][i]])

        for p in pythonList:
            for i in range(0,5):
                if len(p[1]) != 0:
                    writer.writerow([i+1, "python", p[0], p[1][i]])

        for o in optimizationList:
            for i in range(0,5):
                if len(o[1]) != 0:
                    writer.writerow([i+1, "optimized", o[0], o[1][i]])

    file.close()


if __name__ == "__main__":
    main()


from mindt_Selino.Implicant.implicant import *

class BooleanImplicant(Implicant):
    def __init__(self, string):
        self.string = string

    def matches(self, otherImplicant):
    
        difference = 0
        merging = ""

        for z in range(len(self.string)):
            if self.string[z] == otherImplicant.string[z]:
                merging = merging + self.string[z]
            else:
                merging = merging + "-"
                difference+=1

        if(difference == 1):
            return BooleanImplicant(merging)
        
        return None

    def __str__ (self):
        return self.string


from quineMcCluskey import *
from Implicant.intervalmplicant import *

class OptimizedQuineMcCluskey(QuineMcCluskey):
    @classmethod
    def execute(self, input, inputStages):

        #Calculamos suma de mínimos

        input_sumas = []

        for implicant in input:
            minimal_sum = 0

            minimal_sum = sum(interval[0] for interval in implicant.list)
            
            input_sumas.append((minimal_sum, implicant))

        input_sumas.sort(key=lambda x: x[0])

        #Calculamos diferencia máxima

        maxDiff = max(len(sub) for sub in inputStages) - 1

        input = input_sumas   

        output = set()

        while(input):
            auxInput = {element[1]: 0 for element in input}
            mergingSet = set()

            for i in range(0, len(input)-1):
                for j in range(i+1, len(input)):
                    first_sum = input[i][0]
                    second_sum = input[j][0]
                    
                    if(abs(first_sum-second_sum)>maxDiff):
                        break
                        
                    first = input[i][1]
                    second = input[j][1]

                    fusion = first.matches(second)
                        
                    if fusion is not None:
                        minimal_sum = 0
                        minimal_sum = sum(interval[0] for interval in fusion.list)

                        mergingSet.add((minimal_sum, fusion))
                        auxInput[first]=1
                        auxInput[second]=1

            output.update(Resources.primeImplicants(auxInput))
            
            input = list(mergingSet)
            input.sort(key=lambda x: x[0])

        auxOutput = []

        for implicant in list(output):
            var = 0
            auxImplicant = []
            for interval in implicant.list:
                auxImplicant.append((inputStages[var][interval[0]], inputStages[var][interval[1]]))
                var= var+1
            auxOutput.append(IntervalImplicant(auxImplicant))

        return auxOutput


class Resources(object):
    
    @classmethod
    def primeImplicants(self, dicImplicants):
        primeList = []
        for x , y in dicImplicants.items():
            if y == 0:
                primeList.append(x)

        return primeList

    @classmethod
    def printOutput(self, input, output):
        print("input\n", '\n'.join(map(str, input))) 
        print("output\n", '\n'.join(map(str, output)))

    @classmethod
    def listToDictionary(self, list):
        dict = {}

        for element in list:
            dict[element] = 0

        return dict



from dtMinimisation import DtMinimisation

minimization = DtMinimisation("tests/test2/GeneratedExamples/ex.5_3_10")

minimization.printInput()

minimization.printStaggeredInput()

minimization.printOutput()




import sys
import os
import csv
import pandas as pd


def main():

    if(len(sys.argv) < 2):
        print("Please give a test folder:")


    with open("prologTimes.txt") as f:
        prolog = f.read().splitlines() 

    with open("pythonTimes.txt") as f:
        python = f.read().splitlines() 

    times = dict()

    for l in prolog:
        splitted = l.split(".")
        seconds = splitted[2].split("m")[1].split("s")[0].replace(",", ".")
        if times.get(splitted[1]) is None:
            times.update({splitted[1]: [seconds]})
        else:
            times[splitted[1]].append(seconds)

    for l in python:
        splitted = l.split(".")
        seconds = splitted[2].split("m")[1].split("s")[0].replace(",", ".")
        times[splitted[1]].append(seconds)

    tiempos = list(times.items())

    with open('tiempos.csv', mode='w', newline='') as file:
        writer = csv.writer(file)
        writer.writerow(["ejecución", "programa", "ejemplo", "tiempo"])
        for t in tiempos:
            for i in range(0, 5):
                writer.writerow([i+1, "prolog", t[0], t[1][i]])
                writer.writerow([i+1, "python", t[0], t[1][i+5]])

    file.close()

    df = pd.read_csv('tiempos.csv')
    filtered = df.filter(items = ['programa'])

    print(filtered.to_string())

if __name__ == "__main__":
    main()